@@ -16,8 +16,9 @@
 # You should have received a copy of the GNU General Public License
 # along with iggy.  If not, see <http://www.gnu.org/licenses/>.
 # -*- coding: utf-8 -*-
-from pyasp.asp import *
+import sys
 import argparse
+from pyasp.asp import *
 from __iggy__ import query, utils, bioquali
 
 if __name__ == '__main__':
@@ -39,6 +40,10 @@ if __name__ == '__main__':
     parser.add_argument('--no_founded_constraint',
 			help="turn constraints OFF that every variation must be explained by an input, default is ON",
 			action="store_true")
+
+    parser.add_argument('--mics',
+			help="compute minimal inconsistent cores",
+			action="store_true")
 			
     parser.add_argument('--autoinputs',
 			help="compute possible inputs of the network (nodes with indegree 0)",
@@ -55,8 +60,6 @@ if __name__ == '__main__':
 			help="show predictions",
 			action="store_true")
 
-			
-			
 
     args = parser.parse_args()
     
@@ -75,36 +78,51 @@ if __name__ == '__main__':
     print '\nReading network',net_string, '...',
     net = bioquali.readSIFGraph(net_string)
     print 'done.'
-    edge_counter=0
-    activation_counter=0
-    inhibition_counter=0
-    node_counter=0
+    #edge_counter=0
+    activations = set()
+    inhibitions = set()
+    nodes=set()
     for a in net:
-      if a.pred() == 'edge' : edge_counter+=1
+      #if a.pred() == 'edge' : edge_counter+=1
       if a.pred() == 'obs_elabel' :
-	if a.arg(2) == '1' : activation_counter+=1
-        if a.arg(2) == '-1' : inhibition_counter+=1
-      if a.pred() == 'vertex' : node_counter+=1
-    print "   Nodes:",node_counter," Edges:",edge_counter," Activations:",activation_counter," Inhibitions:",inhibition_counter," Unspecified:",(edge_counter-(activation_counter+inhibition_counter))
+	if a.arg(2) == '1' : activations.add((a.arg(0),a.arg(1)))
+        if a.arg(2) == '-1' : inhibitions.add((a.arg(0),a.arg(1)))
+      if a.pred() == 'vertex' : nodes.add(a.arg(0))
+    unspecified = activations & inhibitions
+    print "   Nodes:",len(nodes)," Activations:",len(activations)," Inhibitions:",len(inhibitions)," Dual:",len(unspecified)
 
     print '\nReading observations',obs_string, '...',
     mu = bioquali.readProfile(obs_string)
     print 'done.'
-    count_plus=0
-    count_zero=0
-    count_minus=0
-    count_notminus=0
-    count_notplus=0
-    count_inputs=0
+
+    print '\nChecking observations',obs_string, '...',
+    contradictions = query.get_contradictory_obs(mu)
+    print 'done.'
+    if len(contradictions) == 0 : print '   Observations are consistent.'
+    else:
+      print '   Contradictory observations found. Please correct manualy.'
+      for c in contradictions : print '  ',c
+      utils.clean_up()
+      exit()
+      
+    plus=set()
+    zero=set()
+    minus=set()
+    notminus=set()
+    notplus=set()
+    inputs=set()
     for a in mu:
       if a.pred() == 'obs_vlabel' :
-        if a.arg(2) == '1' : count_plus+=1
-	if a.arg(2) == '0' : count_zero+=1 
-	if a.arg(2) == '-1' : count_minus+=1 
-	if a.arg(2) == 'notMinus' : count_notminus+=1
-	if a.arg(2) == 'notPlus' : count_notplus+=1
-      if a.pred() == 'input' :count_inputs+=1
-    print "   observed +:",count_plus, "observed -:",count_minus," observed 0:",count_zero," observed notPlus:",count_notplus," observed notMinus:",count_notminus," unobserved:",(node_counter-(count_plus+count_minus+count_plus+count_notminus+count_notplus)),' inputs:',count_inputs
+        if a.arg(2) == '1'        : plus.add(a.arg(1))
+	if a.arg(2) == '0'        : zero.add(a.arg(1))
+	if a.arg(2) == '-1'       : minus.add(a.arg(1))
+	if a.arg(2) == 'notMinus' : notminus.add(a.arg(1))
+	if a.arg(2) == 'notPlus'  : notplus.add(a.arg(1))
+      if a.pred() == 'input'      : inputs.add(a.arg(1))
+      
+    unobserved = nodes -(plus|minus|notplus| notminus)
+    not_in_model= (plus|minus|notplus|notminus)-nodes
+    print "   inputs:",len(inputs&nodes)," observed +:",len(plus&nodes), " observed -:",len(minus&nodes)," observed 0:",len(zero&nodes)," observed notPlus:",len(notplus&nodes)," observed notMinus:",len(notminus&nodes)," unobserved:",len(unobserved)," not in model:",len(not_in_model)
 
     if args.autoinputs :
       print '\nComputing input nodes ...',
@@ -120,7 +138,22 @@ if __name__ == '__main__':
       scenfit = query.get_scenfit(net_with_data, LucaConstraint=LC, ConstrainedZero=CZ, FoundedConstraint=FC)
       print 'done.'
       if scenfit == 0 : print "   The network and data are consistent: scenfit = 0."
-      else: print "   The network and data are inconsistent: scenfit = "+str(scenfit)+'.'
+      else:
+	print "   The network and data are inconsistent: scenfit = "+str(scenfit)+'.'
+	
+	if args.mics:
+	  print '\nComputing minimal inconsistent cores (mic\'s) ...',
+	  sys.stdout.flush()
+	  mics = query.get_minimal_inconsistent_cores(net_with_data, LucaConstraint=LC, ConstrainedZero=CZ, FoundedConstraint=FC)
+	  print 'done.'
+	  count = 1
+	  oldmic = 0
+	  for mic in mics:
+	    if oldmic != mic:
+	      print 'mic '+str(count)+':'
+	      utils.print_mic(mic.to_list(),net.to_list(),mu.to_list())
+	      count+=1
+	      oldmic= mic
 
 
       if args.show_colorings >= 0 :
@@ -146,8 +179,22 @@ if __name__ == '__main__':
       mcos = query.get_mcos(net_with_data, LucaConstraint=LC, ConstrainedZero=CZ, FoundedConstraint=FC)
       print 'done.'
       if mcos == 0 : print "   The network and data are consistent: mcos = 0."
-      else: print "   The network and data are inconsistent: mcos = "+str(mcos)+'.'
+      else:
+	print "   The network and data are inconsistent: mcos = "+str(mcos)+'.'
 
+      	if args.mics:
+	  print '\nComputing minimal inconsistent cores (mic\'s) ...',
+	  sys.stdout.flush()
+	  mics = query.get_minimal_inconsistent_cores(net_with_data, LucaConstraint=LC, ConstrainedZero=CZ, FoundedConstraint=FC)
+	  print 'done.'
+	  count = 1
+	  oldmic = 0
+	  for mic in mics:
+	    if oldmic != mic:
+	      print 'mic '+str(count)+':'
+	      utils.print_mic(mic.to_list(),net.to_list(),mu.to_list())
+	      count+=1
+	      oldmic= mic
 
       if args.show_colorings >= 0 :
 	print '\nCompute mcos colorings...',
@@ -168,25 +215,6 @@ if __name__ == '__main__':
 
 
 
-
-
-
-
-      #if args.mics:
-        #print '\nComputing minimal inconsistent cores (mic\'s) ...',
-        #mics = query.get_minimal_inconsistent_cores(net_with_data)
-        #print 'done.'
-        #count = 1
-        #oldmic = 0
-        #for mic in mics:
-          #if oldmic != mic:
-            #print 'mic '+str(count)+':'
-            #utils.print_mic(mic.to_list(),net.to_list(),mu.to_list())
-            #count+=1
-            #oldmic= mic
-
- 
-
     utils.clean_up()
 
 
