@@ -1,5 +1,5 @@
 #!python
-# Copyright (c) 2012, Sven Thiele <sthiele78@gmail.com>
+# Copyright (c) 2014, Sven Thiele <sthiele78@gmail.com>
 #
 # This file is part of iggy.
 #
@@ -69,26 +69,37 @@ if __name__ == '__main__':
     print '\nReading network',net_string, '...',
     net = bioquali.readSIFGraph(net_string)
     print 'done.'
-    edge_counter=0
-    activation_counter=0
-    inhibition_counter=0
-    node_counter=0
+    activations = set()
+    inhibitions = set()
+    nodes=set()
     for a in net:
-      if a.pred() == 'edge' : edge_counter+=1
+      #if a.pred() == 'edge' : edge_counter+=1
       if a.pred() == 'obs_elabel' :
-	if a.arg(2) == '1' : activation_counter+=1
-        if a.arg(2) == '-1' : inhibition_counter+=1
-      if a.pred() == 'vertex' : node_counter+=1
-    print "   Nodes:",node_counter," Edges:",edge_counter," Activations:",activation_counter," Inhibitions:",inhibition_counter," Unspecified:",(edge_counter-(activation_counter+inhibition_counter))
+	if a.arg(2) == '1' : activations.add((a.arg(0),a.arg(1)))
+        if a.arg(2) == '-1' : inhibitions.add((a.arg(0),a.arg(1)))
+      if a.pred() == 'vertex' : nodes.add(a.arg(0))
+    unspecified = activations & inhibitions
+    print "   Nodes:",len(nodes)," Activations:",len(activations)," Inhibitions:",len(inhibitions)," Dual:",len(unspecified)
     
     flist =  os.listdir(obs_dir)
-    print '\nReading',len(flist),'observations from ',obs_dir, '...',
+    print '\nReading',len(flist),'observations from',obs_dir,'...',
     MU = TermSet()
     for f in flist :
       exp= os.path.join(obs_dir,f)
       mu = bioquali.readProfile(exp)
       MU = MU.union(mu)
     print 'done.'
+    print '  ',len(flist),'experiments.'
+
+    print '\nChecking observations ...',
+    contradictions = query.get_contradictory_obs(MU)
+    print 'done.'
+    if len(contradictions) == 0 : print '   Observations are consistent.'
+    else:
+      print '   Contradictory observations found. Please correct manualy.'
+      for c in contradictions : print '  ',c
+      utils.clean_up()
+      exit()
 
     if args.autoinputs :
       print '\nComputing input nodes ...',
