@@ -26,16 +26,16 @@ import tempfile
 from pyasp.asp import *
 
 root = __file__.rsplit('/', 1)[0]
+contradictory_obs_prg =	root +'/encodings/contradictory_obs.lp'
+guess_inputs_prg =     	root + '/encodings/guess_inputs.lp'
 
-guess_inputs_prg =     root + '/encodings/guess_inputs.lp'
+sign_cons_prg   =      	root + '/encodings/sign-cons-3.lp'
+constr_luca_prg =      	root + '/encodings/luca_constraints.lp'
+constr_zero_prg =      	root + '/encodings/constrained_zero.lp'
+founded_prg     =      	root + '/encodings/founded_constraints.lp'
 
-sign_cons_prg   =      root + '/encodings/sign-cons-3.lp'
-constr_luca_prg =      root + '/encodings/luca_constraints.lp'
-constr_zero_prg =      root + '/encodings/constrained_zero.lp'
-founded_prg     =      root + '/encodings/founded_constraints.lp'
-
-keep_inputs_prg =      root + '/encodings/keep_inputs_constraint.lp'
-keep_obs_prg    =      root + '/encodings/keep_observations_constraint.lp'
+keep_inputs_prg =      	root + '/encodings/keep_inputs_constraint.lp'
+keep_obs_prg    =      	root + '/encodings/keep_observations_constraint.lp'
 
 error_measure_prg      =  root + '/encodings/error_measure.lp'
 min_weighted_error_prg =  root + '/encodings/minimize_weighted_error.lp'
@@ -53,7 +53,13 @@ min_removed_edges_prg =   root + '/encodings/minimize_removed_edges.lp'
 min_repairs_prg =         root + '/encodings/minimize_repairs.lp'
 
 
-heu_prg =      root + '/encodings/heuristic.lp'
+mics_prg =		  root + '/encodings/mics.lp'
+mics_constr_luca_prg =    root + '/encodings/mics_luca_constraints.lp'
+mics_constr_zero_prg =    root + '/encodings/mics_constrained_zero.lp'
+mics_founded_prg     =    root + '/encodings/mics_founded_constraints.lp'
+
+
+heu_prg =	          root + '/encodings/heuristic.lp'
 
 show_pred_prg =        root + '/encodings/show_predictions.lp'
 show_labels_prg =      root + '/encodings/show_vlabels.lp'
@@ -72,12 +78,14 @@ def get_scenfit(instance, LucaConstraint=False, ConstrainedZero=True, FoundedCon
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + scenfit + [instance.to_file()]
+    inst = instance.to_file()
+    prg = sem + scenfit + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
     opt = solution[0].score[0]
-    
+
+    os.unlink(inst)
     return opt
     
 def get_scenfit_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -88,8 +96,9 @@ def get_scenfit_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=T
     if LucaConstraint : sem.append(constr_luca_prg)
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
-
-    prg = sem + scenfit + [instance.to_file()]
+    
+    inst = instance.to_file()
+    prg = sem + scenfit + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
@@ -100,7 +109,8 @@ def get_scenfit_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=T
     coptions = str(nm)+' --opt-strategy=5 --opt-mode=optN --opt-bound='+str(opt)
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg,collapseTerms=True,collapseAtoms=False)
-
+    
+    os.unlink(inst)
     return models
      
 def get_predictions_under_scenfit(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -113,7 +123,8 @@ def get_predictions_under_scenfit(instance, LucaConstraint=False, ConstrainedZer
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + scenfit + [instance.to_file()]
+    inst = instance.to_file()
+    prg = sem + scenfit + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
@@ -124,7 +135,8 @@ def get_predictions_under_scenfit(instance, LucaConstraint=False, ConstrainedZer
     coptions = '--opt-strategy=5 --opt-mode=optN --enum-mode=cautious --opt-bound='+str(opt)
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg,collapseTerms=True,collapseAtoms=False)
-    
+
+    os.unlink(inst)    
     return models[0]
 
 def get_mcos(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -137,12 +149,14 @@ def get_mcos(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstr
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + mcos + [instance.to_file()]
+    inst =   instance.to_file()
+    prg = sem + mcos + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
     opt = solution[0].score[0]
-    
+
+    os.unlink(inst) 
     return opt
 
 def get_mcos_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -155,7 +169,8 @@ def get_mcos_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=True
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + mcos + [instance.to_file()]
+    inst =   instance.to_file()
+    prg = sem + mcos + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
@@ -167,6 +182,7 @@ def get_mcos_colorings(instance,nm=1, LucaConstraint=False, ConstrainedZero=True
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg,collapseTerms=True,collapseAtoms=False)
 
+    os.unlink(inst) 
     return models
 
 def get_predictions_under_mcos(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -179,7 +195,8 @@ def get_predictions_under_mcos(instance, LucaConstraint=False, ConstrainedZero=T
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + mcos + [instance.to_file()]
+    inst = instance.to_file()
+    prg = sem + mcos + [inst]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
@@ -191,6 +208,7 @@ def get_predictions_under_mcos(instance, LucaConstraint=False, ConstrainedZero=T
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg,collapseTerms=True,collapseAtoms=False)
 
+    os.unlink(inst)
     return models[0]
 
 
@@ -201,14 +219,15 @@ def get_opt_remove_edges(instance, LucaConstraint=False, ConstrainedZero=True, F
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + scenfit + [remove_edges_prg, min_repairs_prg, instance.to_file() ]
-
+    inst = instance.to_file()
+    prg = sem + scenfit + [remove_edges_prg, min_repairs_prg, inst ]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
     fit = solution[0].score[0]
     repairs = solution[0].score[1]
 
+    os.unlink(inst)
     return (fit,repairs)
 
 
@@ -219,7 +238,8 @@ def get_opt_repairs_remove_edges(instance,nm=1, LucaConstraint=False, Constraine
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + scenfit + [remove_edges_prg, min_repairs_prg, instance.to_file() ]
+    inst = instance.to_file()
+    prg = sem + scenfit + [remove_edges_prg, min_repairs_prg, inst ]
 
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
@@ -232,6 +252,7 @@ def get_opt_repairs_remove_edges(instance,nm=1, LucaConstraint=False, Constraine
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg, collapseTerms=True, collapseAtoms=False)
 
+    os.unlink(inst)
     return models
     
 def get_opt_add_remove_edges(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -240,15 +261,16 @@ def get_opt_add_remove_edges(instance, LucaConstraint=False, ConstrainedZero=Tru
     if LucaConstraint : sem.append(constr_luca_prg)
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
-    
-    prg = sem + scenfit + [remove_edges_prg, add_edges_prg, min_repairs_prg, instance.to_file() ]
 
+    inst = instance.to_file()    
+    prg = sem + scenfit + [remove_edges_prg, add_edges_prg, min_repairs_prg, inst ]
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
     solution = solver.run(prg,collapseTerms=True,collapseAtoms=False)
     fit = solution[0].score[0]
     repairs = solution[0].score[1]
 
+    os.unlink(inst)
     return (fit,repairs)
 
 def get_opt_repairs_add_remove_edges(instance,nm=1, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
@@ -258,7 +280,8 @@ def get_opt_repairs_add_remove_edges(instance,nm=1, LucaConstraint=False, Constr
     if ConstrainedZero : sem.append(constr_zero_prg)
     if FoundedConstraint : sem.append(founded_prg)
 
-    prg = sem + scenfit + [remove_edges_prg, add_edges_prg, min_repairs_prg, instance.to_file() ]
+    inst = instance.to_file()    
+    prg = sem + scenfit + [remove_edges_prg, add_edges_prg, min_repairs_prg, inst ]
 
     coptions = '--opt-strategy=5'
     solver = GringoClasp(clasp_options=coptions)
@@ -271,9 +294,10 @@ def get_opt_repairs_add_remove_edges(instance,nm=1, LucaConstraint=False, Constr
     solver2 = GringoClasp(clasp_options=coptions)
     models = solver2.run(prg, collapseTerms=True, collapseAtoms=False)
 
+    os.unlink(inst)
     return models
     
-def get_minimal_inconsistent_cores(instance,nmodels=0,exclude=[]):
+def get_minimal_inconsistent_cores(instance, LucaConstraint=False, ConstrainedZero=True, FoundedConstraint=True):
     '''
     returns a list containing
     [nmodels] ``TermSet`` objects representing subset minimal inconsistent cores of the system
@@ -281,29 +305,49 @@ def get_minimal_inconsistent_cores(instance,nmodels=0,exclude=[]):
     representing (maybe partial) solutions that are to be avoided. If [nmodels] equals [0]
     then the list contain all feasible models.
     '''
-    inputs = get_reductions(instance)
-    prg = [ dyn_mic_prg, inputs.to_file(), instance.to_file(), exclude_sol(exclude) ] 
-    options=' --heuristic=Vmtf'
-    solver = GringoClaspD(options)
-    models = solver.run(prg,nmodels=0,collapseTerms=True, collapseAtoms=False)
-    os.unlink(prg[1])
-    os.unlink(prg[2])
-    os.unlink(prg[3])
+
+    sem=[mics_prg, heu_prg]
+    if LucaConstraint : sem.append(mics_constr_luca_prg)
+    if ConstrainedZero : sem.append(mics_constr_zero_prg)
+    #if FoundedConstraint : sem.append(mics_founded_prg)
+
+    inst = instance.to_file()   
+    prg = sem+ [inst]
+    coptions='0 --dom-mod=6 --heu=Domain --enum-mode=record'
+    solver = GringoClasp(clasp_options=coptions)
+    models = solver.run(prg,collapseTerms=True, collapseAtoms=False)
+
+    os.unlink(inst)    
     return models
 
 def guess_inputs(instance):
-    prg = [ guess_inputs_prg, instance.to_file() ]
+
+    inst = instance.to_file()   
+    prg = [ guess_inputs_prg, inst ]
     solver = GringoClasp()
     models = solver.run(prg, collapseTerms=True, collapseAtoms=False)
-    os.unlink(prg[1])
+    os.unlink(inst)
     assert(len(models) == 1)
+
+    return models[0]
+
+def get_contradictory_obs(instance):
+
+    inst = instance.to_file()
+    prg = [ contradictory_obs_prg, inst ]
+    solver = GringoClasp()
+    models = solver.run(prg, collapseTerms=True, collapseAtoms=False)
+    os.unlink(inst)
+    assert(len(models) == 1)
+
     return models[0]
 
 def get_reductions(instance):
-    prg = [ reduction_prg, instance.to_file() ]
+    inst = instance.to_file()
+    prg = [ reduction_prg, inst ]
     solver = GringoClasp()
     models = solver.run(prg,0)
-    os.unlink(prg[1])
+    os.unlink(inst)
     assert(len(models) == 1)
     return models[0]
 
