@@ -47,12 +47,12 @@ def print_predictions(predictions) :
     pred_not_minus.difference_update(pred_zero)
     pred_change.difference_update(pred_minus)
     pred_change.difference_update(pred_plus)
-    for p in pred_plus : print('   '+str(p)+ ' = + ')
-    for p in pred_minus : print('   '+str(p)+ ' = - ')
-    for p in pred_zero : print('   '+str(p)+ ' = 0 ')
-    for p in pred_not_plus : print('   '+str(p)+ ' = NOT + ')
-    for p in pred_not_minus : print('   '+str(p)+ ' = NOT - ')
-    for p in pred_change : print('   '+str(p)+ ' = CHANGE ')
+    for p in pred_plus :      print '  ',p,'= +'
+    for p in pred_minus :     print '  ',p,'= -'
+    for p in pred_zero :      print '  ',p,'= 0'
+    for p in pred_not_plus :  print '  ',p,'= NOT +'
+    for p in pred_not_minus : print '  ',p,'= NOT -'
+    for p in pred_change :    print '  ',p,'= CHANGE'
     
     print ' \n   predicted +:',len(pred_plus),
     print ' predicted -:',len(pred_minus),
@@ -62,7 +62,6 @@ def print_predictions(predictions) :
     print ' predicted change:',len(pred_change)
 
     
-
 def print_coloring(colorings) :
     labels = set()
     repairs = set()
@@ -97,29 +96,26 @@ def print_repairs(repairs) :
       
 def print_mic(mic, net, obs):
   
-    nodes = []
+    nodes = set()
     edges = []
-    for node in mic: nodes.append(str(node.arg(1)))
+    for node in mic: nodes.add(node.arg(0))
     
-    predecessors = []
+    predecessors = set()
     for e in net:
        if e.pred() == "obs_elabel" :
-          #print str(e)
-          #print str(e.arg(0)),str(e.arg(1)),str(e.arg(2))
-          if str(e.arg(1)) in nodes : 
-            predecessors.append(str(e.arg(0)))
-            if str(e.arg(2)) == "1" : edges.append( str(e.arg(0))+ " -> " + str(e.arg(1))+ " +")
-            if str(e.arg(2)) == "-1" : edges.append(str(e.arg(0))+ " -> " + str(e.arg(1))+ " -")
-         #TODO ? edges
+          if e.arg(1) in nodes :
+            predecessors.add(e.arg(0))
+            if e.arg(2) == "1"  : edges.append(str(e.arg(0))+" -> "+str(e.arg(1))+" +")
+            if e.arg(2) == "-1" : edges.append(str(e.arg(0))+" -> "+str(e.arg(1))+" -")
     for edge in edges: print('   '+edge)
     for o in obs:
        if o.pred() == "obs_vlabel" :  
-          if str(o.arg(1)) in nodes :
-              if str(o.arg(2))=="1" :  print '   '+str(o.arg(1))+ " = +"
-              if str(o.arg(2))=="-1" :  print '   '+str(o.arg(1))+ " = -"
-          if str(o.arg(1)) in predecessors :
-              if str(o.arg(2))=="1" :  print '   '+str(o.arg(1))+ " = +"
-              if str(o.arg(2))=="-1" :  print '   '+str(o.arg(1))+ " = -"
+          if o.arg(1) in nodes :
+              if o.arg(2) == "1"  :  print '  ', o.arg(1), "= +"
+              if o.arg(2) == "-1" :  print '  ', o.arg(1), "= -"
+          if o.arg(1) in predecessors :
+              if o.arg(2) == "1"  :  print '  ', o.arg(1), "= +"
+              if o.arg(2) == "-1" :  print '  ', o.arg(1), "= -"
     
 
 def clean_up() :
